@@ -21,16 +21,18 @@ public:
 	/* An AGC algorithm must provide the following: */
 	virtual unsigned int getConvergenceFrames() const = 0;
 	virtual std::vector<double> const &getWeights() const = 0;
-	virtual void setEv(double ev) = 0;
+	virtual void setEv(unsigned int channel, double ev) = 0;
 	virtual void setFlickerPeriod(libcamera::utils::Duration flickerPeriod) = 0;
-	virtual void setFixedShutter(libcamera::utils::Duration fixedShutter) = 0;
+	virtual void setFixedShutter(unsigned int channel,
+				     libcamera::utils::Duration fixedShutter) = 0;
 	virtual void setMaxShutter(libcamera::utils::Duration maxShutter) = 0;
-	virtual void setFixedAnalogueGain(double fixedAnalogueGain) = 0;
+	virtual void setFixedAnalogueGain(unsigned int channel, double fixedAnalogueGain) = 0;
 	virtual void setMeteringMode(std::string const &meteringModeName) = 0;
 	virtual void setExposureMode(std::string const &exposureModeName) = 0;
 	virtual void setConstraintMode(std::string const &contraintModeName) = 0;
 	virtual void enableAuto() = 0;
 	virtual void disableAuto() = 0;
+	virtual void setActiveChannels(const std::vector<unsigned int> &activeChannels) = 0;
 };
 
 } /* namespace RPiController */
