@@ -206,8 +206,19 @@ class CommitFile:
 class Commit:
     def __init__(self, commit):
         self.commit = commit
+        self._trailers = []
         self._parse()
 
+    def _parse_trailers(self, lines):
+        for index in range(1, len(lines)):
+            line = lines[index]
+            if not line:
+                break
+
+            self._trailers.append(line)
+
+        return index
+
     def _parse(self):
         # Get the commit title and list of files.
         ret = subprocess.run(['git', 'show', '--format=%s%n%(trailers:only,unfold)', '--name-status',
@@ -217,14 +228,7 @@ class Commit:
 
         self._title = lines[0]
 
-        self._trailers = []
-        for index in range(1, len(lines)):
-            line = lines[index]
-            if not line:
-                break
-
-            self._trailers.append(line)
-
+        index = self._parse_trailers(lines)
         self._files = [CommitFile(f) for f in lines[index:] if f]
 
     def files(self, filter='AMR'):
@@ -266,15 +270,21 @@ class StagedChanges(Commit):
         return parse_diff(diff.splitlines(True))
 
 
-class Amendment(StagedChanges):
+class Amendment(Commit):
     def __init__(self):
-        StagedChanges.__init__(self)
+        Commit.__init__(self, '')
 
     def _parse(self):
-        # Create a title using HEAD commit
-        ret = subprocess.run(['git', 'show', '--pretty=oneline', '--no-patch'],
+        # Create a title using HEAD commit and parse the trailers.
+        ret = subprocess.run(['git', 'show', '--format=%H %s%n%(trailers:only,unfold)',
+                             '--no-patch'],
                              stdout=subprocess.PIPE).stdout.decode('utf-8')
-        self._title = 'Amendment of ' + ret.strip()
+        lines = ret.splitlines()
+
+        self._title = 'Amendment of ' + lines[0].strip()
+
+        self._parse_trailers(lines)
+
         # Extract the list of modified files
         ret = subprocess.run(['git', 'diff', '--staged', '--name-status', 'HEAD~'],
                              stdout=subprocess.PIPE).stdout.decode('utf-8')
@@ -479,7 +489,8 @@ class TrailersChecker(CommitChecker):
         for trailer in commit.trailers:
             match = TrailersChecker.trailer_regex.fullmatch(trailer)
             if not match:
-                raise RuntimeError(f"Malformed commit trailer '{trailer}'")
+                issues.append(CommitIssue(f"Malformed commit trailer '{trailer}'"))
+                continue
 
             key, value = match.groups()
 
