@@ -13,7 +13,7 @@ import sys
 sys.dont_write_bytecode = True
 
 # Make sure that mojom_parser.py can import mojom
-sys.path.append(f'{os.path.dirname(__file__)}/mojo/public/tools/mojom')
+sys.path.insert(0, f'{os.path.dirname(__file__)}/mojo/public/tools/mojom')
 
 import mojo.public.tools.mojom.mojom_parser as parser
 
